@@ -8,7 +8,9 @@ from sqlalchemy_aio.base import AsyncConnection
 
 from ..db import (
     DataBase,
+    DataBaseConnectError,
     DataBaseError,
+    DataBaseQueryError,
     InvalidQueryParameters,
     InvalidQuerySchedule,
     InvalidResultColumnNames,
@@ -358,9 +360,9 @@ class TestDataBase:
 
     @pytest.mark.asyncio
     async def test_connect_error(self):
-        """A DataBaseError is raised if database connection fails."""
+        """A DataBaseConnectError is raised if database connection fails."""
         db = DataBase("db", "sqlite:////invalid")
-        with pytest.raises(DataBaseError) as error:
+        with pytest.raises(DataBaseConnectError) as error:
             await db.connect()
         assert "unable to open database file" in str(error.value)
 
@@ -383,7 +385,7 @@ class TestDataBase:
     async def test_connect_sql_fail(self, caplog):
         """If the SQL at connection fails, an error is raised."""
         db = DataBase("db", "sqlite://", connect_sql=["WRONG"])
-        with caplog.at_level(logging.DEBUG), pytest.raises(DataBaseError) as error:
+        with caplog.at_level(logging.DEBUG), pytest.raises(DataBaseQueryError) as error:
             await db.connect()
         assert not db.connected
         assert 'failed executing query "WRONG"' in str(error.value)
@@ -415,15 +417,18 @@ class TestDataBase:
 
     @pytest.mark.parametrize("connected", [True, False])
     @pytest.mark.asyncio
-    async def test_execute_keep_connected(self, connected):
+    async def test_execute_keep_connected(self, mocker, connected):
         """If keep_connected is set to true, the db is not closed."""
         db = DataBase("db", "sqlite://", keep_connected=connected)
         query = Query(
             "query", ["db"], [QueryMetric("metric", [])], "SELECT 1.0 AS metric"
         )
         await db.connect()
+        mock_conn_detach = mocker.patch.object(db._conn.sync_connection, "detach")
         await db.execute(query)
         assert db.connected == connected
+        if not connected:
+            mock_conn_detach.assert_called_once()
         await db.close()
 
     @pytest.mark.asyncio
@@ -514,7 +519,7 @@ class TestDataBase:
             "SELECT 1 AS metric, 2 AS other",
         )
         await db.connect()
-        with caplog.at_level(logging.ERROR), pytest.raises(DataBaseError) as error:
+        with caplog.at_level(logging.ERROR), pytest.raises(DataBaseQueryError) as error:
             await db.execute(query)
         assert str(error.value) == "Wrong result count from query: expected 1, got 2"
         assert error.value.fatal
@@ -533,7 +538,7 @@ class TestDataBase:
             "SELECT 1 as metric",
         )
         await db.connect()
-        with pytest.raises(DataBaseError) as error:
+        with pytest.raises(DataBaseQueryError) as error:
             await db.execute(query)
         assert str(error.value) == "Wrong result count from query: expected 2, got 1"
         assert error.value.fatal
@@ -548,7 +553,7 @@ class TestDataBase:
             'SELECT 1 AS foo, "bar" AS label',
         )
         await db.connect()
-        with pytest.raises(DataBaseError) as error:
+        with pytest.raises(DataBaseQueryError) as error:
             await db.execute(query)
         assert str(error.value) == "Wrong column names from query"
         assert error.value.fatal
@@ -564,7 +569,7 @@ class TestDataBase:
         )
         mocker.patch.object(db, "_execute_query").side_effect = Exception("boom!")
         await db.connect()
-        with caplog.at_level(logging.DEBUG), pytest.raises(DataBaseError) as error:
+        with caplog.at_level(logging.DEBUG), pytest.raises(DataBaseQueryError) as error:
             await db.execute(query)
         assert str(error.value) == "boom!"
         assert not error.value.fatal
