@@ -2,6 +2,7 @@ import asyncio
 from collections import defaultdict
 from decimal import Decimal
 import logging
+from pathlib import Path
 import re
 
 from prometheus_aioexporter import MetricsRegistry
@@ -92,6 +93,12 @@ def metric_values(metric, by_labels=()):
     return values if by_labels else values[suffix]
 
 
+async def run_queries(db_file: Path, *queries: str):
+    async with DataBase("db", f"sqlite:///{db_file}") as db:
+        for query in queries:
+            await db.execute_sql(query)
+
+
 @pytest.mark.asyncio
 class TestQueryLoop:
     async def test_start(self, query_loop):
@@ -253,7 +260,20 @@ class TestQueryLoop:
         queries_metric = registry.get_metric("database_errors")
         assert metric_values(queries_metric) == [1.0]
 
-    async def test_run_query_increase_error_count(
+    async def test_run_query_increase_database_error_count(
+        self, mocker, query_tracker, config_data, make_query_loop, registry
+    ):
+        """Count of database errors is incremented on failed connection."""
+        query_loop = make_query_loop()
+        db = query_loop._config.databases["db"]
+        mock_connect = mocker.patch.object(db._engine, "connect")
+        mock_connect.side_effect = Exception("connection failed")
+        await query_loop.start()
+        await query_tracker.wait_failures()
+        queries_metric = registry.get_metric("database_errors")
+        assert metric_values(queries_metric) == [1.0]
+
+    async def test_run_query_increase_query_error_count(
         self, query_tracker, config_data, make_query_loop, registry
     ):
         """Count of errored queries is incremented on error."""
@@ -271,11 +291,13 @@ class TestQueryLoop:
         config_data["queries"]["q"]["timeout"] = 0.1
         query_loop = make_query_loop()
         await query_loop.start()
+        db = query_loop._config.databases["db"]
+        await db.connect()
 
         async def execute(sql, parameters):
             await asyncio.sleep(1)  # longer than timeout
 
-        query_loop._config.databases["db"]._conn.execute = execute
+        db._conn.execute = execute
 
         await query_tracker.wait_failures()
         queries_metric = registry.get_metric("queries")
@@ -340,13 +362,15 @@ class TestQueryLoop:
         config_data["queries"]["q"].update(
             {"databases": ["db1", "db2"], "sql": "SELECT * FROM test", "interval": 1.0}
         )
-        async with DataBase("db", f"sqlite:///{db1}") as db:
-            await db.execute_sql("CREATE TABLE test (m INTEGER)")
-            await db.execute_sql("INSERT INTO test VALUES (10)")
+        await run_queries(
+            db1, "CREATE TABLE test (m INTEGER)", "INSERT INTO test VALUES (10)"
+        )
         # the query on the second database returns more columns
-        async with DataBase("db", f"sqlite:///{db2}") as db:
-            await db.execute_sql("CREATE TABLE test (m INTEGER, other INTERGER)")
-            await db.execute_sql("INSERT INTO test VALUES (10, 20)")
+        await run_queries(
+            db2,
+            "CREATE TABLE test (m INTEGER, other INTERGER)",
+            "INSERT INTO test VALUES (10, 20)",
+        )
         query_loop = make_query_loop()
         await query_loop.start()
         await asyncio.sleep(0.1)
@@ -401,13 +425,15 @@ class TestQueryLoop:
                 "interval": None,
             }
         )
-        async with DataBase("db", f"sqlite:///{db1}") as db:
-            await db.execute_sql("CREATE TABLE test (m INTEGER)")
-            await db.execute_sql("INSERT INTO test VALUES (10)")
+        await run_queries(
+            db1, "CREATE TABLE test (m INTEGER)", "INSERT INTO test VALUES (10)"
+        )
         # the query on the second database returns more columns
-        async with DataBase("db", f"sqlite:///{db2}") as db:
-            await db.execute_sql("CREATE TABLE test (m INTEGER, other INTERGER)")
-            await db.execute_sql("INSERT INTO test VALUES (10, 20)")
+        await run_queries(
+            db2,
+            "CREATE TABLE test (m INTEGER, other INTERGER)",
+            "INSERT INTO test VALUES (10, 20)",
+        )
         query_loop = make_query_loop()
         await query_loop.run_aperiodic_queries()
         await query_tracker.wait_failures()
@@ -418,3 +444,48 @@ class TestQueryLoop:
         # succeeding query is run again, failing one is not
         assert len(query_tracker.results) == 2
         assert len(query_tracker.failures) == 1
+
+    async def test_clear_expired_series(
+        self, mocker, tmp_path, query_tracker, config_data, make_query_loop, registry
+    ):
+        """The query loop clears out series last seen earlier than the specified interval."""
+        db = tmp_path / "db.sqlite"
+        config_data["databases"]["db"]["dsn"] = f"sqlite:///{db}"
+        config_data["metrics"]["m"].update(
+            {
+                "labels": ["l"],
+                "expiration": 10,
+            }
+        )
+        # call metric collection directly
+        config_data["queries"]["q"]["sql"] = "SELECT * FROM test"
+        del config_data["queries"]["q"]["interval"]
+
+        await run_queries(
+            db,
+            "CREATE TABLE test (m INTEGER, l TEXT)",
+            'INSERT INTO test VALUES (10, "foo")',
+            'INSERT INTO test VALUES (20, "bar")',
+        )
+        query_loop = make_query_loop()
+        await query_loop.run_aperiodic_queries()
+        await query_tracker.wait_results()
+        queries_metric = registry.get_metric("m")
+        assert metric_values(queries_metric, by_labels=("l",)), {
+            ("foo",): 10.0,
+            ("bar",): 20.0,
+        }
+        # mock that more time has passed than expiration
+        mocker.patch.object(query_loop, "_timestamp").return_value = (
+            query_loop._timestamp() + 20
+        )
+        await run_queries(
+            db,
+            "DELETE FROM test WHERE m = 10",
+        )
+        await query_loop.run_aperiodic_queries()
+        await query_tracker.wait_results()
+        query_loop.clear_expired_series()
+        assert metric_values(queries_metric, by_labels=("l",)), {
+            ("bar",): 20.0,
+        }
